//
//  XMLRPCCallback.m
//  XMLRPC
//
//  Created by Russell Jancewicz on 2012-05-28.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "XMLRPCCallbacks.h"

@implementation XMLRPCBlockCallbacks

@synthesize chain;
@synthesize successCallback;
@synthesize failureCallback;

- (void)request:(XMLRPCRequest *)request didSucceedWithObject:(id)object {
    
    __block id _object = object;
    
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
        successCallback(_object);
    });
    
    if (self.chain && (self.chain != self))     
        [self.chain request:request didSucceedWithObject:object];
}

- (void)request:(XMLRPCRequest *)request didFailWithFaultCode:(NSNumber *)faultCode andFaultString:(NSString *)faultString {
  
    __block NSNumber *_faultCode = faultCode;
    __block NSString *_faultString = faultString;
    
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
        failureCallback(_faultCode, _faultString);
    });
    
    if (self.chain && (self.chain != self))     
        [self.chain request:request didFailWithFaultCode:faultCode andFaultString:faultString];
}


+ (XMLRPCBlockCallbacks *)callbacksWithSuccessCallback:(XMLRPCSuccessBlock)success andFailureCallback:(XMLRPCFailureBlock)failure {
    XMLRPCBlockCallbacks *callbacks = [[XMLRPCBlockCallbacks allocWithZone:NULL] init];
    
    callbacks.successCallback = success;
    callbacks.failureCallback = failure;
    
    return callbacks;
}

+ (XMLRPCBlockCallbacks *)callbacksWithSuccessCallback:(XMLRPCSuccessBlock)success {
    
    return [XMLRPCBlockCallbacks callbacksWithSuccessCallback:success andFailureCallback:nil];
    
}

+ (XMLRPCBlockCallbacks *)callbacksWithFailureCallback:(XMLRPCFailureBlock)failure {
    
    return [XMLRPCBlockCallbacks callbacksWithSuccessCallback:nil andFailureCallback:failure];
}

@end


/*

@implementation XMLRPCSelectorCallbacks

@synthesize successTarget;
@synthesize successCallback;
@synthesize failureTarget;
@synthesize failureCallback;


- (void)request:(XMLRPCRequest *)request didSucceedWithObject:(id)object {
    
}

- (void)request:(XMLRPCRequest *)request didFailWithFaultCode:(NSNumber *)faultCode andFaultString:(NSString *)faultString {
    
}

@end*/