//
//  self.m
//  XMLRPC
//
//  Created by Russell Jancewicz on 2012-05-28.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "XMLRPCEncoder.h"
#import "XMLRPCRequest.h"

@implementation XMLRPCEncoder

+ (id)encoder {
    
    XMLRPCEncoder *encoder = [[XMLRPCEncoder allocWithZone:NULL] init];
    return encoder;
}

- (NSString *)encodeRequest:(XMLRPCRequest *)request {
    
    NSMutableString *xmlrpc = [NSMutableString stringWithString:@"<?xml version=\"1.0\"?><methodCall>"];
    
    [xmlrpc appendFormat: @"<methodName>%@</methodName>", request.method];
    
    [xmlrpc appendString: @"<params>"];
    
    for (id parameter in request.params) {
        [xmlrpc appendFormat:@"<param>%@</param>", [self encodeParameter:parameter]];
    }
    
    [xmlrpc appendString: @"</params>"];
    
    [xmlrpc appendString:@"</methodCall>"];
    
    return xmlrpc;
}

- (NSString *)encodeParameter:(id)parameter {
    
    NSMutableString *xmlrpc = [NSMutableString string];
    
    if (parameter) {
        
        if ([parameter isKindOfClass:[NSArray class]]) {
            [xmlrpc appendString:[self encodeArray:parameter]];
        } else if ([parameter isKindOfClass:[NSDictionary class]]) {
            [xmlrpc appendString:[self encodeDictionary:parameter]];
        } else if ([parameter isKindOfClass:[NSNumber class]]) {
            [xmlrpc appendString:[self encodeNumber:parameter]];
        } else if ([parameter isKindOfClass:[NSString class]]) {
            [xmlrpc appendString:[self encodeString:parameter]];
        } else if ([parameter isKindOfClass:[NSDate class]]) {
            [xmlrpc appendString:[self encodeDate:parameter]];
        } else if ([parameter isKindOfClass:[NSData class]]) {
            // TODO - [xmlrpc appendString:[self encodeData:parameter]];
        } else if (((BOOL)parameter == YES) || ((BOOL)parameter == NO)) {
            [xmlrpc appendString:[self encodeBoolean:(BOOL)parameter]];
        } else {
            // TODO - magial russell function which converts NSObjects into dictionaries.
        }

    }
    
    return xmlrpc;
}

- (NSString *)encodeArray:(NSArray *)array {
    
    NSMutableString *xmlrpc = [NSMutableString stringWithString:@"<value><array><data>"];
    
    for (id element in array)
        [xmlrpc appendString:[self encodeParameter:element]];
    
    [xmlrpc appendString:@"</data></array></value>"];
    
    return xmlrpc;
}

- (NSString *)encodeDictionary:(NSDictionary *)dictionary {
    
    NSMutableString *xmlrpc = [NSMutableString stringWithString:@"<value><struct>"];
    
    for (id key in dictionary)
            [xmlrpc appendFormat:@"<member><name>%@</name><value>%@</value></member>", key, [self encodeParameter:[dictionary objectForKey:key]]];
    
    [xmlrpc appendString:@"</struct></value>"];
    
    return xmlrpc;
}

- (NSString *)encodeNumber:(NSNumber *)number {
    
    NSMutableString *xmlrpc = [NSMutableString stringWithString:@"<value>"];

    //NSString *type = [NSString stringWithCString:[number objCType] encoding:NSUTF8StringEncoding];
    
    // strcmp is faster and cleaner
    
   //todo - add boolean checking 
    //if ( [[number class] isSubclassOfClass:CFBooleanGetTypeID()]) 
          
    
    if ( strcmp("d", [number objCType]) ) {
        [xmlrpc appendFormat:@"<double>%@</double>", number];
    } else {
        [xmlrpc appendFormat:@"<int>%@</int>", number];
    }
    /*
    if ([type isEqualToString:@"d"]) {
        [xmlrpc appendFormat:@"<double>%@</double>", number];
    } else {
        [xmlrpc appendFormat:@"<int>%@</int>", number];
    }*/
    
    [xmlrpc appendString:@"</value>"];
    
    return xmlrpc;
}
   
- (NSString *)encodeString:(NSString *)string {
    
    return [NSString stringWithFormat:@"<value><string>%@</string></value>", string];

}

- (NSString *)encodeDate:(NSDate *)date {
    
    NSDateFormatter *dateFormatter = [[NSDateFormatter allocWithZone:NULL] init];   
    [dateFormatter setDateFormat:@"yyyy-MM-dd'T'HH:mm:ss"];
    
    return [NSString stringWithFormat:@"<value><dateTime.iso8601>%@</dateTime.iso8601></value>", [dateFormatter stringFromDate:date]];

}

- (NSString *)encodeBoolean:(BOOL)boolean {
    
    return [NSMutableString stringWithFormat:@"<value><boolean>%d</boolean></value>", boolean];
    
}

@end
