//
//  XMLRPCRequest.m
//  XMLRPC
//
//  Created by Russell Jancewicz on 2012-05-26.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//




#import "XMLRPCRequest.h"

static NSString *kHTTPMethodPOST = @"POST";
static NSString *kTEXT_XML = @"text/xml";
static NSString *kXMLRPCUserAgent = @"XMLRPC/1.0 (iOS 4.0; OSX 10.5)";

#define NSDebugLog(message) if (DEBUG) NSLog(message)

/* */

@interface XMLRPCRequest ()

@property (strong, nonatomic) NSURLConnection *connection;
@property (readonly, nonatomic) NSMutableData *data;
@property (readonly, nonatomic) NSMutableURLRequest *request;

@end


@implementation XMLRPCRequest

@synthesize method;
@synthesize params;
@synthesize server;
@synthesize connection;
@synthesize callbacks;

@synthesize encoder;
@synthesize delegate;
@synthesize data = _data;   // response data
@synthesize request = _request;

- (id)init {
    
    self = [super init];
    if (self) {
        self.method = @"";
        self.params = [NSArray array];
        self.encoder = [XMLRPCEncoder encoder];
    }
    return self;
}

- (NSURLRequest *)buildRequest {
    
    NSData *body = [[self.encoder encodeRequest:self] dataUsingEncoding:NSUTF8StringEncoding];
    
    _request = [[NSMutableURLRequest allocWithZone:NULL] init];
    
    // timout 30 seconds
    [self.request setTimeoutInterval:30];
    [self.request setHTTPBody:body];
    [self.request setHTTPMethod:kHTTPMethodPOST];
    [self.request setHTTPShouldHandleCookies:YES];
    
    [self.request setValue:kXMLRPCUserAgent forHTTPHeaderField: @"User-Agent"];
    [self.request setValue:kTEXT_XML forHTTPHeaderField:@"Content-Type"];
    [self.request setValue:kTEXT_XML forHTTPHeaderField:@"Accept"];
    [self.request setValue:[NSString stringWithFormat:@"%d", [body length]] forHTTPHeaderField:@"Content-Length"];

    [self.request setURL:self.server];
    
    if (!self.server) 
        _request = nil;
    
    return self.request;

}

- (void)sendSynchronously {
    
    NSURLRequest *request = [self buildRequest];
    NSURLResponse *response = nil;
    NSError *error = nil;
    
    _data = [NSMutableData dataWithData:[NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error]];
    
    if (error) {
        [self connection:nil didFailWithError:error];
    } else {
        [self connectionDidFinishLoading:nil];
    }
}

- (void)sendAsynchronously {
    
    if (self.connection == nil) {
        self.connection = [NSURLConnection connectionWithRequest:[self buildRequest] delegate:self];
        
        if (self.connection) {
            _data = [NSMutableData data];
            [self.data setLength:0];
        }
    }
    
    [self.connection start];
    
}

- (void)send {
    [self sendAsynchronously];
}


/* NSURLConnection Methods */


- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    
}

- (BOOL)connectionShouldUseCredentialStorage:(NSURLConnection *)connection {
    return YES;
}

- (void)connection:(NSURLConnection *)connection willSendRequestForAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge {
    
    if ([challenge.protectionSpace.authenticationMethod isEqualToString:NSURLAuthenticationMethodServerTrust]) {
        
        NSURLCredential *certificate = [NSURLCredential credentialForTrust:challenge.protectionSpace.serverTrust];
        [challenge.sender useCredential:certificate forAuthenticationChallenge:challenge];
        
    }
    
}

- (NSCachedURLResponse *)connection:(NSURLConnection *)connection willCacheResponse:(NSCachedURLResponse *)cachedResponse {
    return nil;
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
 
    // todo - NSArray *cookies = [[NSHTTPCookieStorage sharedHTTPCookieStorage] cookies];
    
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    [self.data appendData:data];
}

- (void)connection:(NSURLConnection *)connection didSendBodyData:(NSInteger)bytesWritten totalBytesWritten:(NSInteger)totalBytesWritten totalBytesExpectedToWrite:(NSInteger)totalBytesExpectedToWrite {
}

- (NSURLRequest *)connection:(NSURLConnection *)connection willSendRequest:(NSURLRequest *)request redirectResponse:(NSURLResponse *)redirectResponse {
    return request;
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
        
    if (self.data && self.data.length > 0) {
            
        XMLRPCResponse *response = [XMLRPCResponse responseWithData:self.data forRequest:self];
        
        if (self.callbacks) {
            [response performCallbacks];
        } else if (self.delegate) {
            
            SEL selector = @selector(xmlrpcDidCompleteRequest:withResponse:);
            
            /* 
                CLANG complains of a potential leak however we will not leak so ignore it
             */
            
            if ([self.delegate respondsToSelector:selector])
                #pragma clang diagnostic push
                #pragma clang diagnostic ignored "-Warc-performSelector-leaks"
                [self.delegate performSelector:selector];
                #pragma clang diagnostic pop
    
        } else {
            
        }
    }
    
    [self.connection cancel];
    
}



@end






